# -*- coding: utf8 -*-

from django.conf import settings
from django.core.cache import cache
from django.db import models
from django.template.defaultfilters import slugify

from os import makedirs, listdir
from os.path import join, isfile
from shutil import rmtree
from PIL import Image
from datetime import datetime
from gestion import fields

DOSSIER_EVENEMENTS = "evenements"
NOM_AFFICHE = "affiche.png"
NOM_AFFICHE_MINI = "affiche_mini.png"
TAILLE_MINI = (128, 128)


def liste_evts_avec_videos():
    liste = cache.get("live_evts_avec_videos")
    if liste:
        return liste

    liste = []

    for element in listdir(join(settings.MEDIA_ROOT, DOSSIER_EVENEMENTS)):
        if isfile(join(settings.MEDIA_ROOT, DOSSIER_EVENEMENTS, element, "video.m4v")):
            liste.append(element)

    cache.set("live_evts_avec_videos", liste, 3600)

    return liste


class Evenement(models.Model):
    nom_interne = models.SlugField(max_length=128, editable=False, unique=True)
    nom = models.CharField(
        max_length=128,
        verbose_name="Nom",
        help_text="Le nom de l’événement: JT1, Show Clubs…",
    )
    description = models.CharField(
        max_length=128,
        blank=True,
        verbose_name="Description",
        help_text="Un petit commentaire sur l’événement…",
    )
    date_debut = fields.DateTimeField(verbose_name="Date de début")
    date_effective = fields.DateTimeField(verbose_name="Date effective", editable=False)
    prive = models.BooleanField(
        verbose_name="Evénement privé",
        default=True,
        help_text="Les événements privés ne sont pas accessible aux utilisateurs non connectés.",
    )
    est_cloture = models.BooleanField(editable=False, default=False)

    def enreg_image(self, fichier):
        fichier.seek(0)

        image = Image.open(fichier)
        image.save(
            join(settings.MEDIA_ROOT, DOSSIER_EVENEMENTS, self.nom_interne, NOM_AFFICHE),
            format="png",
        )
        image.thumbnail(TAILLE_MINI, Image.ANTIALIAS)
        image.save(
            join(
                settings.MEDIA_ROOT,
                DOSSIER_EVENEMENTS,
                self.nom_interne,
                NOM_AFFICHE_MINI,
            ),
            format="png",
        )

    def url_affiche(self):
        # C’est une URL donc c’est forcément des /
        return DOSSIER_EVENEMENTS + "/" + self.nom_interne + "/" + NOM_AFFICHE

    def url_affiche_mini(self):
        # C’est une URL donc c’est forcément des /
        return DOSSIER_EVENEMENTS + "/" + self.nom_interne + "/" + NOM_AFFICHE_MINI

    def url_video_m4v(self):
        # C’est une URL donc c’est forcément des /
        return DOSSIER_EVENEMENTS + "/" + self.nom_interne + "/video.m4v"

    def url_video_ogg(self):
        # C’est une URL donc c’est forcément des /
        return DOSSIER_EVENEMENTS + "/" + self.nom_interne + "/video.ogg"

    def url_video_webm(self):
        # C’est une URL donc c’est forcément des /
        return DOSSIER_EVENEMENTS + "/" + self.nom_interne + "/video.webm"

    def chemin_base(self):
        return join(settings.MEDIA_ROOT, DOSSIER_EVENEMENTS, self.nom_interne)

    def delta(self):
        delta = datetime.today() - self.date_debut
        return delta.seconds + delta.days * 24 * 3600

    def delta_retard(self):
        delta = datetime.today() - self.date_effective
        return delta.seconds + delta.days * 24 * 3600

    def attente(self):
        delta = self.delta()
        if delta >= 0:
            msg = "Retard compris, le live commencera dans "

            delta = self.delta_retard()

            if delta > 0:
                return "???"

            delta = abs(delta)

            if delta >= 60:
                return f"{msg}{delta / 60} minutes, {delta % 60} secondes…"

            return f"{msg}{delta} secondes…"

        delta = abs(delta)

        if delta >= 3600:
            return f"Le live commencera dans {delta / 3600} heures, {(delta % 3600) / 60} minutes…"

        if delta >= 60:
            return f"Le live commencera dans {delta / 60} minutes, {delta % 60} secondes…"

        return f"Le live commencera dans {delta} secondes…"

    def save(self, *args, **kwargs):
        if not self.nom_interne:
            self.nom_interne = self.date_debut.strftime("%Y-%m-%d") + "_" + slugify(self.nom)
            makedirs(join(settings.MEDIA_ROOT, DOSSIER_EVENEMENTS, self.nom_interne))

        if not self.date_effective or self.date_effective < self.date_debut:
            self.date_effective = self.date_debut
        super(Evenement, self).save(*args, **kwargs)

    def delete(self):
        rmtree(join(settings.MEDIA_ROOT, DOSSIER_EVENEMENTS, self.nom_interne), True)

        super(Evenement, self).delete()

    def __str__(self):
        return self.nom

    class Meta:
        unique_together = (("nom", "date_debut"),)
        ordering = ["-date_debut", "nom"]
